@@ -2,11 +2,13 @@
 
 use std::collections::HashMap;
 
-use anyhow::anyhow;
 use futures::stream::*;
 use itertools::Itertools;
 use log::*;
 use maplit::*;
+use pact_models::http_utils::HttpAuth;
+use pact_models::pact::{load_pact_from_json, Pact};
+use pact_models::PACT_RUST_VERSION;
 use regex::{Captures, Regex};
 use reqwest::{Method, RequestBuilder};
 use serde::{Deserialize, Serialize};
@@ -15,11 +17,6 @@ use serde_with::skip_serializing_none;
 use tokio::time::{Duration, sleep};
 
 use pact_matching::Mismatch;
-use pact_models::http_utils::HttpAuth;
-use pact_models::message_pact::MessagePact;
-use pact_models::pact::Pact;
-use pact_models::PACT_RUST_VERSION;
-use pact_models::sync_pact::RequestResponsePact;
 
 use crate::MismatchResult;
 
@@ -563,7 +560,7 @@ pub async fn fetch_pacts_from_broker(
   broker_url: &str,
   provider_name: &str,
   auth: Option<HttpAuth>
-) -> anyhow::Result<Vec<anyhow::Result<(Box<dyn Pact + Send>, Option<PactVerificationContext>, Vec<Link>)>>> {
+) -> anyhow::Result<Vec<anyhow::Result<(Box<dyn Pact + Send + Sync>, Option<PactVerificationContext>, Vec<Link>)>>> {
   trace!("fetch_pacts_from_broker(broker_url='{}', provider_name='{}', auth={})", broker_url,
     provider_name, auth.clone().unwrap_or_default());
 
@@ -611,16 +608,8 @@ pub async fn fetch_pacts_from_broker(
             Ok((pact_link, pact_json)) => {
               let href = pact_link.href.unwrap_or_default();
               let links = links_from_json(&pact_json);
-              match pact_json {
-                Value::Object(ref map) => if map.contains_key("messages") {
-                  MessagePact::from_json(&href, &pact_json)
-                    .map(|pact| (pact.boxed(), None, links))
-                } else {
-                  RequestResponsePact::from_json(&href, &pact_json)
-                    .map(|pact| (pact.boxed(), None, links))
-                },
-                _ => Err(anyhow!("Link '{}' does not point to a valid pact file", href))
-              }
+              load_pact_from_json(href.as_str(), &pact_json)
+                .map(|pact| (pact, None, links))
             },
             Err(err) => Err(err.into())
           }
@@ -639,9 +628,10 @@ pub async fn fetch_pacts_dynamically_from_broker(
   pending: bool,
   include_wip_pacts_since: Option<String>,
   provider_tags: Vec<String>,
+  provider_branch: Option<String>,
   consumer_version_selectors: Vec<ConsumerVersionSelector>,
   auth: Option<HttpAuth>
-) -> Result<Vec<Result<(Box<dyn Pact + Send>, Option<PactVerificationContext>, Vec<Link>), PactBrokerError>>, PactBrokerError> {
+) -> Result<Vec<Result<(Box<dyn Pact + Send + Sync>, Option<PactVerificationContext>, Vec<Link>), PactBrokerError>>, PactBrokerError> {
   trace!("fetch_pacts_dynamically_from_broker(broker_url='{}', provider_name='{}', pending={}, \
     include_wip_pacts_since={:?}, provider_tags: {:?}, consumer_version_selectors: {:?}, auth={})",
     broker_url, provider_name, pending, include_wip_pacts_since, provider_tags,
@@ -665,6 +655,7 @@ pub async fn fetch_pacts_dynamically_from_broker(
     // Construct the Pacts for verification payload
     let pacts_for_verification = PactsForVerificationRequest {
       provider_version_tags: provider_tags,
+      provider_version_branch: provider_branch,
       include_wip_pacts_since,
       consumer_version_selectors,
       include_pending_status: pending,
@@ -689,7 +680,8 @@ pub async fn fetch_pacts_dynamically_from_broker(
     // Find all of the Pact links
     let pact_links = match response {
       Some(v) => {
-        let pfv: PactsForVerificationResponse = serde_json::from_value(v).unwrap_or(PactsForVerificationResponse { embedded: PactsForVerificationBody { pacts: vec!() } });
+        let pfv: PactsForVerificationResponse = serde_json::from_value(v)
+          .unwrap_or(PactsForVerificationResponse { embedded: PactsForVerificationBody { pacts: vec!() } });
 
         if pfv.embedded.pacts.len() == 0 {
           return Err(PactBrokerError::NotFound(format!("No pacts were found for this provider")))
@@ -748,20 +740,9 @@ pub async fn fetch_pacts_dynamically_from_broker(
           Ok((pact_link, pact_json, context)) => {
             let href = pact_link.href.unwrap_or_default();
             let links = links_from_json(&pact_json);
-            match pact_json {
-              Value::Object(ref map) => if map.contains_key("messages") {
-                match MessagePact::from_json(&href, &pact_json) {
-                  Ok(pact) => Ok((pact.boxed(), Some(context), links)),
-                  Err(err) => Err(PactBrokerError::ContentError(format!("{}", err)))
-                }
-              } else {
-                match RequestResponsePact::from_json(&href, &pact_json) {
-                  Ok(pact) => Ok((pact.boxed(), Some(context), links)),
-                  Err(err) => Err(PactBrokerError::ContentError(format!("{}", err)))
-                }
-              },
-              _ => Err(PactBrokerError::ContentError(format!("Link '{}' does not point to a valid pact file", href)))
-            }
+            load_pact_from_json(href.as_str(), &pact_json)
+              .map(|pact| (pact, Some(context), links))
+              .map_err(|err| PactBrokerError::ContentError(format!("{}", err)))
           },
           Err(err) => Err(err)
         }
@@ -799,10 +780,15 @@ pub async fn publish_verification_results(
   result: TestResult,
   version: String,
   build_url: Option<String>,
-  provider_tags: Vec<String>
+  provider_tags: Vec<String>,
+  branch: Option<String>
 ) -> Result<serde_json::Value, PactBrokerError> {
   let hal_client = HALClient::with_url(broker_url, auth.clone());
 
+  if branch.is_some() {
+    publish_provider_branch(&hal_client, &links, &branch.unwrap(), &version).await?;
+  }
+
   if !provider_tags.is_empty() {
     publish_provider_tags(&hal_client, &links, provider_tags, &version).await?;
   }
@@ -951,6 +937,33 @@ async fn publish_provider_tags(
   }
 }
 
+async fn publish_provider_branch(
+  hal_client: &HALClient,
+  links: &[Link],
+  branch: &str,
+  version: &str) -> Result<(), PactBrokerError> {
+  let hal_client = hal_client.clone().with_doc_context(links)?
+    .navigate("pb:provider", &hashmap!{}).await?;
+
+    match hal_client.find_link("pb:branch-version") {
+    Ok(link) => {
+      let template_values = hashmap! {
+        "branch".to_string() => branch.to_string(),
+        "version".to_string() => version.to_string(),
+      };
+      match hal_client.clone().put_json(hal_client.clone().parse_link_url(&link, &template_values)?.as_str(), "{}").await {
+        Ok(_) => debug!("Pushed branch {} for provider version {}", branch, version),
+        Err(err) => {
+          error!("Failed to push branch {} for provider version {}", branch, version);
+          return Err(err);
+        }
+      }
+      Ok(())
+    },
+    Err(_) => Err(PactBrokerError::LinkError("Can't publish provider branch as there is no 'pb:branch-version' link. Please ugrade to Pact Broker version 2.86.0 or later for branch support".to_string()))
+  }
+}
+
 #[skip_serializing_none]
 #[derive(Serialize, Deserialize, Debug, Clone)]
 #[serde(rename_all = "camelCase")]
@@ -1012,7 +1025,9 @@ pub struct PactsForVerificationRequest {
   /// Find WIP pacts after given date
   pub include_wip_pacts_since: Option<String>,
   /// Detailed pact selection criteria , see https://docs.pact.io/pact_broker/advanced_topics/consumer_version_selectors/
-  pub consumer_version_selectors: Vec<ConsumerVersionSelector>
+  pub consumer_version_selectors: Vec<ConsumerVersionSelector>,
+  /// Current provider version branch if used (instead of tags)
+  pub provider_version_branch: Option<String>
 }
 
 #[skip_serializing_none]
@@ -1043,12 +1058,13 @@ mod tests {
   use env_logger::*;
   use expectest::expect;
   use expectest::prelude::*;
+  use pact_models::{Consumer, PactSpecification, Provider};
+  use pact_models::prelude::RequestResponsePact;
+  use pact_models::sync_interaction::RequestResponseInteraction;
 
   use pact_consumer::*;
   use pact_consumer::prelude::*;
   use pact_matching::Mismatch::MethodMismatch;
-  use pact_models::{Consumer, PactSpecification, Provider};
-  use pact_models::sync_interaction::RequestResponseInteraction;
 
   use super::*;
   use super::{content_type, json_content_type};
@@ -1062,11 +1078,13 @@ mod tests {
   #[tokio::test]
   async fn fetch_returns_an_error_if_it_does_not_get_a_success_response() {
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBroker")
-        .interaction("a request to a non-existant path", |i| {
+        .interaction("a request to a non-existant path", "", |mut i| {
             i.given("the pact broker has a valid pact");
             i.request.path("/hello");
             i.response.status(404);
+            futures::future::ready(i)
         })
+        .await
         .start_mock_server();
 
     let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1078,12 +1096,14 @@ mod tests {
   #[tokio::test]
   async fn fetch_returns_an_error_if_it_does_not_get_a_hal_response() {
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-      .interaction("a request to a non-json resource", |i| {
+      .interaction("a request to a non-json resource", "", |mut i| {
           i.request.path("/nonjson");
           i.response
               .header("Content-Type", "text/html")
               .body("<html></html>");
+          futures::future::ready(i)
       })
+      .await
       .start_mock_server();
 
     let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1131,16 +1151,20 @@ mod tests {
     #[tokio::test]
     async fn fetch_returns_an_error_if_it_does_not_get_a_valid_hal_response() {
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a non-hal resource", |i| {
+            .interaction("a request to a non-hal resource", "", |mut i| {
                 i.request.path("/nonhal");
                 i.response.header("Content-Type", "application/hal+json");
+                futures::future::ready(i)
             })
-            .interaction("a request to a non-hal resource 2", |i| {
+            .await
+            .interaction("a request to a non-hal resource 2", "", |mut i| {
                 i.request.path("/nonhal2");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("<html>This is not JSON</html>");
+                futures::future::ready(i)
             })
+            .await
             .start_mock_server();
 
         let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1157,11 +1181,13 @@ mod tests {
   async fn fetch_retries_the_request_on_50x_errors() {
     let _ = env_logger::try_init();
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-      .interaction("a request to a hal resource", |i| {
+      .interaction("a request to a hal resource", "", |mut i| async move {
         i.given("server returns a gateway error");
         i.request.path("/");
         i.response.status(503);
+        i
       })
+      .await
       .start_mock_server();
 
     let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1175,11 +1201,13 @@ mod tests {
   async fn post_json_retries_the_request_on_50x_errors() {
     let _ = env_logger::try_init();
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-      .interaction("a POST request", |i| {
+      .interaction("a POST request", "", |mut i| async move {
         i.given("server returns a gateway error");
         i.request.path("/").method("POST");
         i.response.status(503);
+        i
       })
+      .await
       .start_mock_server();
 
     let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1193,11 +1221,13 @@ mod tests {
   async fn put_json_retries_the_request_on_50x_errors() {
     let _ = env_logger::try_init();
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-      .interaction("a PUT request", |i| {
+      .interaction("a PUT request", "", |mut i| async move {
         i.given("server returns a gateway error");
         i.request.path("/").method("PUT");
         i.response.status(503);
+        i
       })
+      .await
       .start_mock_server();
 
     let client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1241,12 +1271,14 @@ mod tests {
     async fn fetch_link_returns_an_error_if_the_previous_resource_was_not_hal() {
       try_init().unwrap_or(());
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a non-hal json resource", |i| {
+            .interaction("a request to a non-hal json resource", "", |mut i| async move {
                 i.request.path("/");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("{}");
+                i
             })
+            .await
             .start_mock_server();
 
         let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1262,12 +1294,14 @@ mod tests {
     async fn fetch_link_returns_an_error_if_the_previous_resource_links_are_not_correctly_formed() {
       try_init().unwrap_or(());
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a hal resource with invalid links", |i| {
+            .interaction("a request to a hal resource with invalid links", "", |mut i| async move {
                 i.request.path("/");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("{\"_links\":[{\"next\":{\"href\":\"abc\"}},{\"prev\":{\"href\":\"def\"}}]}");
+                i
             })
+            .await
             .start_mock_server();
 
         let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1282,12 +1316,14 @@ mod tests {
   #[tokio::test]
   async fn fetch_link_returns_an_error_if_the_previous_resource_does_not_have_the_link() {
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-        .interaction("a request to a hal resource", |i| {
+        .interaction("a request to a hal resource", "", |mut i| async move {
             i.request.path("/");
             i.response
                 .header("Content-Type", "application/hal+json")
                 .body("{\"_links\":{\"next\":{\"href\":\"/abc\"},\"prev\":{\"href\":\"/def\"}}}");
+            i
         })
+        .await
         .start_mock_server();
 
     let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1302,18 +1338,22 @@ mod tests {
     #[tokio::test]
     async fn fetch_link_returns_the_resource_for_the_link() {
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a hal resource", |i| {
+            .interaction("a request to a hal resource", "", |mut i| async move {
                 i.request.path("/");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("{\"_links\":{\"next\":{\"href\":\"/abc\"},\"prev\":{\"href\":\"/def\"}}}");
+                i
             })
-            .interaction("a request to next", |i| {
+            .await
+            .interaction("a request to next", "", |mut i| async move {
                 i.request.path("/abc");
                 i.response
                     .header("Content-Type", "application/json")
                     .json_body(json_pattern!("Yay! You found your way here"));
+                i
             })
+            .await
             .start_mock_server();
 
         let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1326,20 +1366,24 @@ mod tests {
 
     #[tokio::test]
     async fn fetch_link_returns_handles_absolute_resource_links() {
-      try_init().unwrap_or(());
+        try_init().unwrap_or(());
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a hal resource with absolute paths", |i| {
+            .interaction("a request to a hal resource with absolute paths", "", |mut i| async move {
                 i.request.path("/");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("{\"_links\":{\"next\":{\"href\":\"http://localhost/abc\"},\"prev\":{\"href\":\"http://localhost/def\"}}}");
+                i
             })
-            .interaction("a request to next", |i| {
+            .await
+            .interaction("a request to next", "", |mut i| async move {
                 i.request.path("/abc");
                 i.response
                     .header("Content-Type", "application/json")
                     .json_body(json_pattern!("Yay! You found your way here"));
+                i
             })
+            .await
             .start_mock_server();
 
         let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1354,19 +1398,22 @@ mod tests {
     async fn fetch_link_returns_the_resource_for_the_templated_link() {
       try_init().unwrap_or(());
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBrokerStub")
-            .interaction("a request to a templated hal resource", |i| {
+            .interaction("a request to a templated hal resource", "", |mut i| async move {
                 i.request.path("/");
                 i.response
                     .header("Content-Type", "application/hal+json")
                     .body("{\"_links\":{\"document\":{\"href\":\"/doc/{id}\",\"templated\":true}}}");
-
+                i
             })
-            .interaction("a request for a document", |i| {
+            .await
+            .interaction("a request for a document", "", |mut i| async move {
                 i.request.path("/doc/abc");
                 i.response
                     .header("Content-Type", "application/json")
                     .json_body(json_pattern!("Yay! You found your way here"));
+                i
             })
+            .await
             .start_mock_server();
 
         let mut client = HALClient::with_url(pact_broker.url().as_str(), None);
@@ -1381,7 +1428,7 @@ mod tests {
     async fn fetch_pacts_from_broker_returns_empty_list_if_there_are_no_pacts() {
       try_init().unwrap_or(());
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBroker")
-            .interaction("a request to the pact broker root", |i| {
+            .interaction("a request to the pact broker root", "", |mut i| async move {
                 i.request
                     .path("/")
                     .header("Accept", "application/hal+json")
@@ -1396,15 +1443,19 @@ mod tests {
                             }
                         }
                     }));
+                i
             })
-            .interaction("a request for a providers pacts", |i| {
+            .await
+            .interaction("a request for a providers pacts", "", |mut i| async move {
                 i.given("There are no pacts in the pact broker");
                 i.request
                     .path("/pacts/provider/sad_provider/latest")
                     .header("Accept", "application/hal+json")
                     .header("Accept", "application/json");
                 i.response.status(404);
+                i
             })
+            .await
             .start_mock_server();
 
         let result = fetch_pacts_from_broker(pact_broker.url().as_str(),
@@ -1433,7 +1484,7 @@ mod tests {
             .. RequestResponsePact::default() }
             .to_json(PactSpecification::V3).unwrap().to_string();
         let pact_broker = PactBuilder::new("RustPactVerifier", "PactBroker")
-            .interaction("a request to the pact broker root", |i| {
+            .interaction("a request to the pact broker root", "", |mut i| async move {
                 i.request
                     .path("/")
                     .header("Accept", "application/hal+json")
@@ -1448,8 +1499,10 @@ mod tests {
                             }
                         }
                     }));
+                i
             })
-            .interaction("a request for a providers pacts", |i| {
+            .await
+            .interaction("a request for a providers pacts", "", |mut i| async move {
                 i.given("There are two pacts in the pact broker");
                 i.request
                     .path("/pacts/provider/happy_provider/latest")
@@ -1465,8 +1518,10 @@ mod tests {
                             ]
                         }
                     }));
+                i
             })
-            .interaction("a request for the first provider pact", |i| {
+            .await
+            .interaction("a request for the first provider pact", "", |mut i| async move {
                 i.given("There are two pacts in the pact broker");
                 i.request
                     .path("/pacts/provider/happy_provider/consumer/Consumer/version/1.0.0")
@@ -1475,8 +1530,10 @@ mod tests {
                 i.response
                     .header("Content-Type", "application/json")
                     .body(pact.clone());
+                i
             })
-            .interaction("a request for the second provider pact", |i| {
+            .await
+            .interaction("a request for the second provider pact", "", |mut i| async move {
                 i.given("There are two pacts in the pact broker");
                 i.request
                     .path("/pacts/provider/happy_provider/consumer/Consumer2/version/1.0.0")
@@ -1485,7 +1542,9 @@ mod tests {
                 i.response
                     .header("Content-Type", "application/json")
                     .body(pact2.clone());
+                i
             })
+            .await
             .start_mock_server();
 
         let result = fetch_pacts_from_broker(pact_broker.url().as_str(),
@@ -1514,7 +1573,7 @@ mod tests {
         .to_json(PactSpecification::V3).unwrap().to_string();
 
       let pact_broker = PactBuilder::new("RustPactVerifier", "PactBroker")
-          .interaction("a request to the pact broker root", |i| {
+          .interaction("a request to the pact broker root", "", |mut i| async move {
             i.given("Pacts for verification is enabled");
             i.request
               .path("/")
@@ -1531,8 +1590,10 @@ mod tests {
                     }
                   }
               }));
+            i
           })
-          .interaction("a request to the pacts for verification endpoint", |i| {
+          .await
+          .interaction("a request to the pacts for verification endpoint", "", |mut i| async move {
             i.given("There are pacts to be verified");
             i.request
               .get()
@@ -1549,8 +1610,10 @@ mod tests {
                     }
                 }
             }));
+            i
           })
-          .interaction("a request to fetch pacts to be verified", |i| {
+          .await
+          .interaction("a request to fetch pacts to be verified", "", |mut i| async move {
             i.given("There are pacts to be verified");
             i.request
               .post()
@@ -1563,6 +1626,7 @@ mod tests {
                 }),
                 "providerVersionTags": each_like!("master"),
                 "includePendingStatus": like!(false),
+                "providerVersionBranch": like!("main")
               }));
             i.response
               .header("Content-Type", "application/hal+json")
@@ -1598,8 +1662,10 @@ mod tests {
                   }
                 }
               }));
+            i
         })
-        .interaction("a request for a pact by version", |i| {
+        .await
+        .interaction("a request for a pact by version", "", |mut i| async move {
           i.given("There is a pact with version 12345678");
           i.request
             .path("/pacts/provider/happy_provider/consumer/Consumer/pact-version/12345678")
@@ -1608,36 +1674,38 @@ mod tests {
           i.response
             .header("Content-Type", "application/json")
             .body(pact.clone());
+          i
       })
+      .await
       .start_mock_server();
 
-    let result = fetch_pacts_dynamically_from_broker(pact_broker.url().as_str(), "happy_provider".to_string(), false, None, vec!("master".to_string()), vec!(ConsumerVersionSelector {
-      consumer: None,
-      tag: Some("prod".to_string()),
-      fallback_tag: None,
-      latest: None,
-      branch: None,
-      deployed_or_released: None,
-      deployed: None,
-      released: None,
-      main_branch: None,
-      environment: None,
-    }), None).await;
+      let result = fetch_pacts_dynamically_from_broker(pact_broker.url().as_str(), "happy_provider".to_string(), false, None, vec!("master".to_string()), Some("main".to_string()), vec!(ConsumerVersionSelector {
+        consumer: None,
+        tag: Some("prod".to_string()),
+        fallback_tag: None,
+        latest: None,
+        branch: None,
+        deployed_or_released: None,
+        deployed: None,
+        released: None,
+        main_branch: None,
+        environment: None,
+      }), None).await;
 
-    match &result {
-      Ok(_) => (),
-      Err(err) => panic!("Expected an Ok result, got a error {}", err)
-    }
-
-    let pacts = &result.unwrap();
-    expect!(pacts.len()).to(be_equal_to(1));
-
-    for pact in pacts {
-      match pact {
+      match &result {
         Ok(_) => (),
         Err(err) => panic!("Expected an Ok result, got a error {}", err)
       }
-    }
+
+      let pacts = &result.unwrap();
+      expect!(pacts.len()).to(be_equal_to(1));
+
+      for pact in pacts {
+        match pact {
+          Ok(_) => (),
+          Err(err) => panic!("Expected an Ok result, got a error {}", err)
+        }
+      }
   }
 
   #[tokio::test]
@@ -1645,7 +1713,7 @@ mod tests {
     try_init().unwrap_or(());
 
     let pact_broker = PactBuilder::new("RustPactVerifier", "PactBroker")
-      .interaction("a request to the pact broker root", |i| {
+      .interaction("a request to the pact broker root", "", |mut i| async move {
         i.given("Pacts for verification is enabled");
         i.request
           .path("/")
@@ -1662,8 +1730,10 @@ mod tests {
                 }
               }
           }));
+        i
       })
-      .interaction("a request to the pacts for verification endpoint", |i| {
+      .await
+      .interaction("a request to the pacts for verification endpoint", "", |mut i| async move {
         i.request
           .get()
           .path("/pacts/provider/sad_provider/for-verification")
@@ -1679,8 +1749,10 @@ mod tests {
                 }
             }
         }));
+        i
       })
-      .interaction("a request to fetch pacts to be verified", |i| {
+      .await
+      .interaction("a request to fetch pacts to be verified", "", |mut i| async move {
         i.given("There are no pacts to be verified");
         i.request
           .post()
@@ -1693,6 +1765,7 @@ mod tests {
             }),
             "providerVersionTags": each_like!("master"),
             "includePendingStatus": like!(false),
+            "providerVersionBranch": like!("main")
           }));
         i.response
           .json_body(json_pattern!({
@@ -1700,10 +1773,12 @@ mod tests {
                 "pacts": []
               }
             }));
+        i
       })
+    .await
     .start_mock_server();
 
-    let result = fetch_pacts_dynamically_from_broker(pact_broker.url().as_str(), "sad_provider".to_string(), false, None, vec!("master".to_string()), vec!(ConsumerVersionSelector {
+    let result = fetch_pacts_dynamically_from_broker(pact_broker.url().as_str(), "sad_provider".to_string(), false, None, vec!("master".to_string()), Some("main".to_string()), vec!(ConsumerVersionSelector {
       consumer: None,
       tag: Some("prod".to_string()),
       fallback_tag: None,
